Attribute VB_Name = "modFolderTree"
Option Explicit
Option Compare Text
Option Base 0

Sub BeginFolderList()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' BeginFolderList
' This procedure sets up some variable value and the calls the DoFolderTree procedure, which
' does all the work, recurively calling itself for subfolders. To automatically create
' a directory tree, you should use the DoFolderTree procedure directly, passing in the
' appropriate variables.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim TopFolderName As String             ' name of staring folder
Dim TopFolderObj As Scripting.Folder    ' folder object of starting folder
Dim DestRange As Range                  ' first cell of listing
Dim Indent As Boolean                   ' ident results
Dim ListFiles As Boolean                ' list files in addition to folders
Dim SystemAndHidden As Boolean          ' list system and hidden files/folders
Dim FSO As Scripting.FileSystemObject   ' FileSystemObject
Dim FullPaths As Boolean                ' always list full paths
Dim BoldFolders As Boolean              ' Bold folder names
''''''''''''''''''''''''''''''''''''''
' Change these values as desired
''''''''''''''''''''''''''''''''''''''
TopFolderName = "C:\Test"
Set DestRange = ThisWorkbook.Worksheets("Sheet1").Range("A1")
Indent = True
SystemAndHidden = True
ListFiles = True
FullPaths = True
BoldFolders = True
'''''''''''''''''''''''''''''''''''''
' End Of Options
'''''''''''''''''''''''''''''''''''''

If SystemAndHidden = True Then
    If Dir(TopFolderName, vbDirectory + vbNormal + vbSystem + vbHidden) = vbNullString Then
        MsgBox "The folder: " & vbCrLf & _
            TopFolderName & vbCrLf & _
            "cannot be found."
        Exit Sub
    End If
Else
    If Dir(TopFolderName, vbNormal + vbDirectory) = vbNullString Then
        MsgBox "The folder: " & vbCrLf & _
            TopFolderName & vbCrLf & _
            "cannot be found. It may be a system or hidden folder."
        Exit Sub
    End If
End If

Set FSO = New Scripting.FileSystemObject
Set TopFolderObj = FSO.GetFolder(TopFolderName)

'''''''''''''''''''''''''''''''''''''''''''''''''''''
' Cal DoFolderTree, which does all the work. Your
' code should call this procedure directly.
'''''''''''''''''''''''''''''''''''''''''''''''''''''
DoFolderTree FSO:=FSO, FolderObj:=TopFolderObj, Dest:=DestRange, _
            ListFiles:=ListFiles, Indent:=Indent, _
            SystemAndHidden:=SystemAndHidden, FullPaths:=FullPaths, _
            BoldFolders:=BoldFolders
            
            




End Sub


Sub DoFolderTree(FSO As Scripting.FileSystemObject, FolderObj As Scripting.Folder, _
    Dest As Range, ListFiles As Boolean, Indent As Boolean, _
    SystemAndHidden As Boolean, FullPaths As Boolean, BoldFolders As Boolean)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' DoFolderTree
' This procedure creates the folder tree, starting in the cell referenced by Dest.
'   FSO                 is a reference to an existing Scripting.FileSystemObject.
'
'   FolderObj           is a Scripting.Folder object for which the tree should be built.
'
'   Dest                is a range reference where the listing should begin.
'
'   ListFiles           indicates whether files within folders should be listed.
'
'   Ident               indicates whether the results should be indented.
'
'   SystemAndHidden     indicates whether to list hidden and system files/folders.
'
'   FullPaths           indicates whether to list the full path of the file/folder
'
'   BoldFolders         indicates whether to bold folder names.
'
' This procedure will call itself recursively for subfolders of FolderObj. There is no
' limitation on the depth of nesting of subfolders.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim SF As Scripting.Folder
Dim OneFile As Scripting.File

If FullPaths = True Then
    Dest.Value = FolderObj.Path
Else
    Dest.Value = FolderObj.Name
End If
Dest.Font.Bold = BoldFolders

''''''''''''''''''''''''''''''''''''''''''''
' Process each subfolder of FolderObj,
' calling this same procedure recursively.
''''''''''''''''''''''''''''''''''''''''''''
For Each SF In FolderObj.SubFolders
    '''''''''''''''''''''''''''''''''''''''''''
    ' Test whether we're listing hidden and/or
    ' system folders and proceed accordingly.
    '''''''''''''''''''''''''''''''''''''''''''
    If (SystemAndHidden = True) Or _
            ((SystemAndHidden = False) And (IsSystemOrHidden(SF.Path) = False)) Then
        If Indent = True Then
            Set Dest = Dest(2, 2)
        Else
            Set Dest = Dest(2, 1)
        End If
        ''''''''''''''''''''''''''''''''''''''''''''
        ' Here, the procedure calls itself, passing
        ' the parameter values that were passed in,
        ' but passing the SubFolder taken from the
        ' For Each loop.
        ''''''''''''''''''''''''''''''''''''''''''''
        DoFolderTree FSO:=FSO, FolderObj:=SF, Dest:=Dest, ListFiles:=ListFiles, _
            Indent:=Indent, SystemAndHidden:=SystemAndHidden, _
            FullPaths:=FullPaths, BoldFolders:=BoldFolders
    End If
Next SF

''''''''''''''''''''''''''''''''''''
' If we're listing files, list them
' here. This code appears after the
' code that deals with subfolder, so
' files in a folder are always listed
' after subfolders of a folder.
'''''''''''''''''''''''''''''''''''
If ListFiles = True Then
    If Indent = True Then
        Set Dest = Dest(1, 2)
    End If
    For Each OneFile In FolderObj.Files
        If (SystemAndHidden = True) Or _
            ((SystemAndHidden = False) And (IsSystemOrHidden(OneFile.Path) = False)) Then
            Set Dest = Dest(2, 1)
            Dest.Font.Bold = False
            If FullPaths = True Then
                Dest.Value = OneFile.Path
            Else
                Dest.Value = OneFile.Name
            End If
        End If
    Next OneFile
    If Indent = True Then
        If Dest.Column > 1 Then
            Set Dest = Dest(1, 0)
        End If
    End If
End If

'''''''''''''''''''''''''''''''''
' If we're indenting, we have to
' move the Dest range back one
' column to the left.
'''''''''''''''''''''''''''''''''
If Dest.Column > 1 Then
    If Indent = True Then
        Set Dest = Dest(1, 0)
    End If
End If

End Sub

Function IsSystemOrHidden(FullName As String) As Boolean
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' IsSystemOrHidden
' This returns True or False indicating whether the file or folder
' named in FullName is a system or hidden file/folder.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Attr As VbFileAttribute
Attr = GetAttr(FullName)
If (Attr And vbSystem) Or (Attr And vbHidden) Then
    IsSystemOrHidden = True
Else
    IsSystemOrHidden = False
End If


End Function
